@@ -17,10 +17,6 @@
  * Author: Stef Walter <stefw@collabora.co.uk>
  */
 
-#if !defined (__GCR_H_INSIDE__) && !defined (GCR_COMPILATION)
-#error "Only <gcr/gcr.h> can be included directly."
-#endif
-
 #ifndef GCR_GNUPG_RECORDS_H
 #define GCR_GNUPG_RECORDS_H
 
@@ -43,8 +39,6 @@ gboolean            _gcr_gnupg_records_parse_user_id         (const gchar *user_
                                                               gchar **email,
                                                               gchar **comment);
 
-GIcon *             _gcr_gnupg_records_get_icon              (GPtrArray *records);
-
 G_END_DECLS
 
 #endif /* __GCR_GNUPG_RECORD_H__ */
